#!/usr/bin/perl -w

# Author: Andreas Schmid

use strict;
use warnings;
use Switch;
use JSON;
use LWP::UserAgent;
use HTTP::Cookies;
use Log::Log4perl;


#INIT AND GET LOGGER
Log::Log4perl->init("ISACMonitoringEventhandler-log.conf");
my $logger = Log::Log4perl->get_logger("ISACMonitoringEventhandler");

#TODO - move credentials to configfile
#CREDENTIALS
my $user   = 'demo@internetx.com';
my $secret = 'test#11!';

#TODO - remove api var ---> is only necessary for development
#DEV/LIVE
my $API = 2;    # 1 is LIVE, 2 is DEV, 3 is DEV_LOCAL

#TODO - move api url to configfile
#API MONITORING URL
my $API_URL_LIVE		= "";
my $API_URL_DEV			= "http://resellercloud-dev.isac.de:8080/ISAC-API/rest/monitoring/notification";
my $API_URL_DEV_LOCAL 	= "http://127.0.0.1:8080/ISAC-API/rest/monitoring/notification";

#TODO - move oauth url to configfile
#API OAUTH URL
my $OAUTH_URL_LIVE      = "";
my $OAUTH_URL_DEV       = "http://resellercloud-dev.isac.de:8080/ISAC-API/oauth/token";
my $OAUTH_URL_DEV_LOCAL = "http://127.0.0.1:8080/ISAC-API/oauth/token";

#UserAgent
my $ua         = undef;
my $cookie_jar = undef;


#ICINGA MACROS
#http://docs.icinga.org/latest/de/macrolist.html

#Icinga/Nagios environment vars

#HOST ENV
my $HOSTNAME    = $ENV{'HOSTNAME'};
my $HOSTADDRESS = $ENV{'HOSTADDRESS'};
my $HOSTSTATE   = $ENV{'HOSTSTATE'};
my $HOSTOUTPUT  = $ENV{'HOSTOUTPUT'};


#SERVICE ENV
my $SERVICESTATE  = $ENV{'SERVICESTATE'};
my $SERVICEOUTPUT = $ENV{'SERVICEOUTPUT'};



#START MAIN
main();

sub main {
#TODO - remove switch ---> is only necessary for development
	switch ($API) {

		case (1) {
			handleRequest( $API_URL_LIVE, getOAuthToken($OAUTH_URL_LIVE) );
		}
		case (2) {
			handleRequest( $API_URL_DEV, getOAuthToken($OAUTH_URL_DEV) );
		}
		case (3) {
			handleRequest( $API_URL_DEV_LOCAL, getOAuthToken($OAUTH_URL_DEV_LOCAL) );
		}

	}

}

sub handleRequest {
	my ( $url, $token ) = @_;

	$logger->info( "trying to handle monitoring request with token " . $token );

	# Create authorization header with fresh token
	my $header = HTTP::Headers->new( 'Authorization' => 'Bearer ' . $token );

	# Create a request
	my $req = HTTP::Request->new( POST => $url, $header );
	$req->content_type('application/x-www-form-urlencoded');
	$req->content( "ip=$HOSTADDRESS&"
		  . "notification_event=$HOSTSTATE$SERVICESTATE&"
		  . "monitoring_message=$HOSTOUTPUT$SERVICEOUTPUT&");
	
	$logger->info("trying to send monitoring event to " . $url);
	# Pass request to the user agent and get a response back
	my $res = $ua->request($req);
	
	# Check the outcome of the response
	if ( $res->is_success ) {
		$logger->info("monitoring message successfully transmitted!");
		$logger->info( $res->content );
	}
	else {
		$logger->error("error on request!");
		$logger->error( $res->status_line );
		exit 1;
	}

}

sub getOAuthToken {
	my ($url) = @_;

	$logger->info( "trying to get OAuth-Token from " . $url );

	# Create a user agent object
	$ua = LWP::UserAgent->new;
	$ua->agent("ISAC-MonitoringEventhandler/1.0 ");

#TODO - check if this is necessary
#	$cookie_jar = HTTP::Cookies->new(
#		file           => "cookies.lwp",
#		ignore_discard => 1,
#		autosave       => 1,
#	);

	$cookie_jar = HTTP::Cookies->new();
	$ua->cookie_jar($cookie_jar);

#TODO - move appKey devKey to configfile
	# Create a request
	my $req = HTTP::Request->new( POST => $url );
	$req->content_type('application/x-www-form-urlencoded');
	$req->content( "grant_type=client_credentials&"
		  . "client_id=$user&"
		  . "client_secret=$secret&"
		  . "devKey=377f276e-8646-4d0a-bb31-10c75a27608b&"
		  . "appKey=1cb65a7c-7c78-4fcd-a1b3-73b841f211e2" );

	# Pass request to the user agent and get a response back
	my $res = $ua->request($req);

	# Check the outcome of the response
	if ( $res->is_success ) {
		$logger->info("sucessfully retrived oauth token!");
		$logger->info( $res->content );
		my $responseHash = from_json( $res->content );
		return $responseHash->{"access_token"};
	}
	else {
		$logger->error("error on getting oauth token...");
		$logger->error( $res->status_line );
		exit 1;
	}

}
